import processing.pdf.*;

int maxAgents = 100;  // The number of agents that start on the screen
Agent[] bubble = new Agent[maxAgents];

void setup() {
  colorMode(RGB);
  size(595, 841);  // A-Format
  background(255);
  smooth();
  strokeWeight(1);  // Minimum for stroke weight.
  
  int xPos, yPos, maxBubbles;
  color bubbleColor;
  
  for (int i=0; i<maxAgents; i++) {    
    xPos = (int)random(10,width-10);  // Random horizontal location on screen, with in a margin of 10 pixels.
    yPos = (int)random(10,height-10);  // Random vertical location.
    bubbleColor = color(random(255),random(255),random(255)); // Random color for a bubble trail.
    maxBubbles = (int)random(50,1000); // Number of bubbles an agent can produce.
    
    bubble[i] = new Agent(xPos, yPos, bubbleColor, maxBubbles);  // Input all generated data into an agent.
  }
   beginRecord(PDF, "Final.pdf");  // Open a PDF named Final.pdf and start recording.
}

void draw() {  
  color clrFrom, clrTo, clrBetween;
  float xFrom, yFrom, xTo, yTo, xBetween, yBetween;
  int maxLife = 0;
  
  for (int i=0; i<maxAgents; i++) {
    
    if (bubble[i].life > 0) {
      bubble[i].update();  // Assign new location for every agent.
      
      // If two bubbles are within a distance of 30 pixels, the
      // two merge into one new bubble, with their averege color
      // and placed right in the middle of the two bubbles. The  
      // new bubble will have the combined lives of the two merging.
      for (int j=0; j<maxAgents; j++) {
        if (dist(bubble[i].xF, bubble[i].yF, bubble[j].xF, bubble[j].yF) < 30 && i != j && bubble[j].life > 0) {
          clrFrom = bubble[i].clr;
          clrTo = bubble[j].clr;
          xFrom = bubble[i].xF;
          xTo = bubble[j].xF;
          yFrom = bubble[i].yF;
          yTo = bubble[j].yF;
        
          clrBetween = lerpColor(clrFrom, clrTo, .50);
          xBetween = lerp(xFrom, xTo, .50);
          yBetween = lerp(yFrom, yTo, .50);

          bubble[i].life = bubble[i].life + bubble[j].life;
          bubble[i].clr = clrBetween;
          bubble[i].a = bubble[i].a + bubble[j].a;
          bubble[j].life = 0;
        
          break;
        }
      }
    }
  }
  
  // Count the number of agents still alive.
  maxLife = 0;
  for (int k = 0; k < maxAgents; k++) {
    if (bubble[k].life > 0) {
      maxLife++;
    }
  }
  
  // If there are less then 12 (of 100) surviving agents, 
  // then end the program and write the PDF.   
  if (maxLife < 12) { 
    endRecord();
    exit();
  }
}
